// $Id: queue.c,v 1.5 2002/07/30 16:15:10 blues Exp $

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* ChangeLog {{{
 * $Log: queue.c,v $
 * Revision 1.5  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.4  2002/07/20 20:03:04  blues
 * - automatic action taken by vim
 *
 * Revision 1.3  2002/06/30 07:51:00  blues
 * - switched to our main web page
 *
 * Revision 1.2  2002/06/30 07:36:12  blues
 * - cosmetics
 *
 * Revision 1.1  2002/05/18 23:28:24  ventox
 * - initial revision
 * }}} */

// include for PCH file
#include "pch.h"

// include local header files
#include "mutex.h"
#include "queue.h"

#if defined( _DEBUG )
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif /* _DEBUG */

/* private functions */

/* _vqueue_lock() {{{ */
static _status_t _vqueue_lock( _vqueue_t * _queue ) {
	_status_t result = OK;
	ASSERT( _queue );
	
	if ( (*_queue) == NULL ) {
		result = invalid_argument;
	}
	else {
		result = _vmutex_ACQUIRE( (*_queue)->_mutex );
	}
	
	return result;
} /* }}} */

/* _vqueue_unlock() {{{ */
static _status_t _vqueue_unlock( _vqueue_t * _queue ) {
	_status_t result = OK;
	ASSERT( _queue );
	
	if ( (*_queue) == NULL ) {
		result = invalid_argument;
	}
	else {
		result = _vmutex_RELEASE( (*_queue)->_mutex );
	}
	
	return result;
} /* }}} */

/* implementation */

/* _vqueue_init() - function creates and initializes a queue object {{{ */
_status_t _vqueue_init( _vqueue_t * _queue ) {
	_status_t result = OK;
	_vqueue_t queue;
	
	ASSERT( _queue );
	
	queue = (_vqueue_t)
		calloc( 1, sizeof( struct _tagQUEUE ) );
	
	if ( queue == NULL ) {
		result = out_of_memory;
	}
	else {
		_vmutex_init( &queue->_mutex, FALSE );
	}
	
	if ( result == OK )
		(*_queue) = queue;
	
	return result;
} /* }}} */

/* _vqueue_destroy() - function cleans up and destroys the specyfied queue object {{{ */
_status_t _vqueue_destroy( _vqueue_t * _queue, _vqueue_cleanup_t _cleanup ) {
	_status_t result = OK;
	ASSERT( _queue );
	
	if ( (*_queue) == NULL ) {
		result = invalid_argument;
	}
	else {
		_vqueue_item_t _t, _n;        
		
		/* begin critical section block */
		_vqueue_lock( _queue );                
		
		_t = (*_queue)->_head;
		_n = NULL;          
		(*_queue)->_head = NULL;
		(*_queue)->_tail = NULL;        
		
		_vqueue_unlock( _queue );
		/* end critical section block   */
		
		while ( _t != NULL ) {
			_n = _t->_next;
			if ( _cleanup != NULL )	{
				_cleanup( _t->_item );
			}
			
			free( (void *)_t );
			_t = _n;
		}
		
		_vmutex_DESTROY( 
				(*_queue)->_mutex );
		
		free( (void *)(*_queue) ); 
		(*_queue) = NULL;
	}
	
	return result;
} /* }}} */

/* _vqueue_pop() - function returns the first element in the specyfied queue {{{ */
void * _vqueue_pop( _vqueue_t * _queue ) {
	void * result = NULL;
	ASSERT( _queue );
	
	if ( (*_queue) != NULL ) {
		/* begin critical section block */
		_vqueue_lock( _queue );
		
		if ( (*_queue)->_head ) {
			_vqueue_item_t _t = 
				(*_queue)->_head;
			
			result = _t->_item;
			(*_queue)->_head = _t->_next;
			
			free( (void *)_t );
		}
		
		if ( (*_queue)->_head == NULL ) {
			(*_queue)->_tail = NULL;
		}
		
		_vqueue_unlock( _queue );
		/* end critical section block   */
	}
	
	return result;
} /* }}} */

/* _vqueue_peek() - function returns the first element in the specyfied queue {{{ */
void * _vqueue_peek( _vqueue_t * _queue ) {
	void * result = NULL;
	ASSERT( _queue );
	
	if ( (*_queue) != NULL ) {
		_vqueue_lock( _queue );
		
		if ( (*_queue)->_head )
			result = (*_queue)->_head->_item;
		
		_vqueue_unlock( _queue );
	}
	
	return result;
} /* }}} */

/* _vqueue_put() - function puts an element in the end of the specyfied queue {{{ */
_status_t _vqueue_put( _vqueue_t * _queue, void * _item ) {
	_status_t result = OK;
	ASSERT( _queue );
	
	if ( (*_queue) == NULL ) {
		result = invalid_argument;
	}
	else {
		_vqueue_item_t _t;
		
		/* begin critical section block */
		_vqueue_lock( _queue );
		
		_t = (_vqueue_item_t)
			calloc( 1, sizeof( struct _tagQUEUE_ITEM ) );
		
		if ( _t == NULL ) {
			return out_of_memory;
		}
		
		_t->_item = _item;
		if ( (*_queue)->_tail ) {
			(*_queue)->_tail->_next = _t;
		}
		
		(*_queue)->_tail = _t;
		
		if ( (*_queue)->_head == NULL ) {
			(*_queue)->_head = _t;
		}
		
		_vqueue_unlock( _queue );
		/* end critical section block   */
	}
	
	return result;
} /* }}} */

/* _vqueue_clear() {{{ */
_status_t _vqueue_clear( _vqueue_t * _queue, _vqueue_cleanup_t _cleanup ) {
	_status_t result = OK;
	ASSERT( _queue );
	
	if ( (*_queue) == NULL ) {
		result = invalid_argument;
	}
	else {
		_vqueue_item_t _t, _n;        
		
		/* begin critical section block */
		
		_vqueue_lock( _queue );
		
		_t = (*_queue)->_head;
		_n = NULL;
		(*_queue)->_head = NULL;
		(*_queue)->_tail = NULL;
		
		_vqueue_unlock( _queue );
		/* end critical section block   */
		
		while ( _t != NULL ) {
			_n = _t->_next;
			if ( _cleanup != NULL ) {
				_cleanup( _t->_item );
			}
			
			free( (void *)_t );
			_t = _n;
		}
	}
	
	return result;
} /* }}} */

/* _vqueue_is_empty() {{{ */
BOOL _vqueue_is_empty( _vqueue_t * _queue ) {
	ASSERT( _queue );
	
	if ( (*_queue) == NULL )
		return TRUE;

	return (BOOL)((*_queue)->_head == NULL);
} /* }}} */




/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

